///
/// @file        can_imx6.h
/// @copyright   Copyright (c) 2015 Toradex AG
/// $Author: raja.sivasankaran $
/// $Rev: 3184 $
/// $Date: 2016-04-25 12:48:30 +0530 (Thu, 25 Apr 2016) $
/// @brief       Library to support CAN on iMX6 modules
/// @test        Colibri iMX6 and Apalis iMX6
/// @target      Colibri iMX6 and Apalis iMX6.
///
/// @anchor imx6can_pins
/// <h2 class = "groupheader">CAN Pin Availability on Colibri iMX6 and Apalis iMX6 modules</h2>
/// The tables below list the module pins, on which CAN functionality is available.\n
///
/// <h3>Colibri iMX6</h3>
/// The SODIMM pins used by the library, if no particular pin specification is given, are marked _italic_.
/// |   Function    | "CAN1"         | "CAN2"     |
/// |---------------|----------------|------------|
/// |      RX       | _63_\n 186     | _188_      |
/// |      TX       | _55_\n 184     | _178_      |
///
/// Please note that these pins are used for different functions in the Colibri standard pinout definition.
///
/// <h3>Apalis iMX6</h3>
/// MXM3 Pins defined in the standard Apalis pinout are marked __bold__.
/// |   Function    | "CAN1"       | "CAN2"    |
/// |---------------|--------------|-----------|
/// |      RX       | __12__\n     | __16__\n  |
/// |      TX       | __14__\n     | __18__\n  |
///
/// @page pcfg_can CAN Configuration Options
/// @tableofcontents
///
/// @section scfg_can_imx6 iMX6 CAN Configurations
/// @anchor acfg_can_imx6
/// The following table explains the configuration parameters supported by calls of
/// * Imx6Can_SetConfigString()
/// * Imx6Can_GetConfigString()
/// * Imx6Can_SetConfigInt()
/// * Imx6Can_GetConfigInt()
///
/// For a list of compatible parameters, refer to @ref acfg_can "Compatible CAN Configurations".
/// <table class="libParam" >
///   <tr><th>                              Parameter Name </th><th> Value    </th><th> Description</th></tr>
///   <tr><td class="paramname" rowspan="2">Implementation </td><td> FlexCAN  </td><td> Internal iMX6 CAN controller.</td></tr>
///   <tr>                                                      <td> MCP2515  </td><td> External SPI based MCP2515 CAN controller.</td></tr>
///   <tr><td class="paramname" rowspan="6">Interface      </td><td> CAN1     </td><td> iMX6 internal CAN1 controller.</td></tr>
///   <tr>                                                      <td> CAN2     </td><td> iMX6 internal CAN2 controller.</td></tr>
///   <tr>                                                      <td> SPI1<sup><a href="#fn2">2</a></sup>     </td><td> SPI channel 1 used to interface MCP2515 controller.</td></tr>
///   <tr>                                                      <td> SPI2<sup><a href="#fn2">2</a></sup>     </td><td> SPI channel 2 used to interface MCP2515 controller.</td></tr>
///   <tr>                                                      <td> SPI3<sup><a href="#fn2">2</a></sup>     </td><td> SPI channel 3 used to interface MCP2515 controller.</td></tr>
///   <tr>                                                      <td> SPI4<sup><a href="#fn2">2</a></sup>     </td><td> SPI channel 4 used to interface MCP2515 controller.</td></tr>
///   <tr><td class="paramname"            >ioRx<sup><a href="#fn5">5</a></sup>             </td><td> DWORD   </td><td> GPIO used as CAN RX pin in uIo format.</td></tr>
///   <tr><td class="paramname"            >ioTx<sup><a href="#fn5">5</a></sup>             </td><td> DWORD   </td><td> GPIO used as CAN TX pin in uIo format.</td></tr>
///   <tr><td class="paramname"            >BitRateHz      </td><td> DWORD    </td><td> CAN speed in Hz.</td></tr>
///   <tr><td class="paramname"            >Timeout        </td><td> DWORD    </td><td> CAN timeout in ms, INFINITE to wait until operation is complete.</td></tr>
///   <tr><td class="paramname" rowspan="2">FrameFormat                                   </td><td> standard </td><td> 11 bit frame ID.</td></tr>
///   <tr>                                                                                 <td> extended </td><td> 29 bit frame ID.</td></tr>
///   <tr><td class="paramname" rowspan="2">RtrFormat                                   </td><td> data </td><td> RTR data frame.</td></tr>
///   <tr>                                                                              <td> remote </td><td> RTR remote frame.</td></tr>
///   <tr><td class="paramname"            >FilterId
///                                         <sup><a href="#fn1">1</a>,<a href="#fn2">2</a></sup>
///                                                        </td><td> DWORD    </td><td> Frame ID to filter.</td></tr></td></tr>
///   <tr><td class="paramname"            >FilterMask
///                                         <sup><a href="#fn1">1</a>,<a href="#fn2">2</a></sup>
///                                                        </td><td> DWORD    </td><td> Frame ID mask to filter.</td></tr></td></tr>
///   <tr><td class="paramname" rowspan="2">FilterRemote
///                                         <sup><a href="#fn1">1</a></sup><sup>,<a href="#fn2">2</a></sup>
///                                                        </td><td> remote     </td><td> Filter messages for remote frame.</td></tr>
///   <tr>                                                      <td> none    </td><td> Data and remote frame receive.</td></tr>
///   <tr><td class="paramname"            >SpiHandle
///                                         <sup><a href="#fn3">3</a>,<a href="#fn4">4</a></sup>
///                                                        </td><td> DWORD    </td><td> HANDLE to SPI channel used to interface MCP2515 controller, obtained from Spi_Init().</td></tr>
///   <tr><td class="paramname" rowspan="3">BusStatus<sup><a href="#fn4">4</a></sup>
///                                                        </td><td> 0  </td><td> CAN controller is active, without any restrictions</td></tr>
///   <tr>                                                      <td> 1  </td><td> Only Messages and passive error frames may be transmitted.</td></tr>
///   <tr>                                                      <td> 2  </td><td> Messages can neither be transmitted nor received.</td></tr>
/// </table>
/// <sup><ol>
///   <li name="fn1" value=1>Frame filter logic: (FilterID) & (FilterMask) == (received frame ID) & (FilterMask)<br>
///      Any received frame ID that matches with the filter will be returned from CAN read function.
///   <li name="fn2" value="2">This feature is not yet supported. The precise definition of the API is subject to change.
///   <li name="fn3" value=3>Only relevant if "Implementation" is configured to "MCP2515"
///   <li name="fn4" value=4>Read-Only. Get this parameter <b>after</b> calling Can_Open()<br>
///                          Feature is not yet supported in iMX6 Image version 1.0 ( April, 2016)
///   <li name="fn5" value=5>Set these parameters <b>before</b> calling Can_Open()
/// </ol></sup>

#ifndef __CAN_IMX6_H__
#define __CAN_IMX6_H__

#include "can.h"
#include "TdxCommon.h"

#ifdef __cplusplus
extern "C" {
#endif

//******************************************************************************
/// Initialize the CAN channel.\n
/// This function call does not modify any hardware registers. Therefore the
/// returned handle can be used to make additional configuration settings, which
/// will become active when calling Imx6Can_Open().
/// @param[in]    portName        CAN channel name. E.g. "CAN1", "SPI2"
/// @return                       Handle to the CAN port. NULL if the function failed.
HANDLE Imx6Can_Init(const TCHAR *portName);

//******************************************************************************
/// De-initialize the HANDLE. The port must be closed before.
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_Deinit(HANDLE hPort);

//******************************************************************************
/// Apply configuration to the hardware.
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_Open(HANDLE hPort);

//******************************************************************************
/// Close CAN
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_Close(HANDLE hPort);

//******************************************************************************
/// Read one CAN message.\n
/// Can messages are buffered in a receive buffer. This function reads one message from the buffer.
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @param[out]   msg              Buffer to store the received message.
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_Read(HANDLE hPort, pCanMsg msg);

//******************************************************************************
/// Write one CAN Message.\n
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @param[in]    msg              Pointer to the message to be sent
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_Write(HANDLE hPort, pCanMsg msg);

//******************************************************************************
/// Retrieves a DWORD configuration parameter.
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @param[in]    paramName        Parameter value to retrieve\n
///                                Refer to the @ref scfg_can_imx6 for a list of supported parameters.
/// @param[out]   pValue           Data obtained
/// @return                        Number of bytes read. 0 if paramName not found.
DWORD Imx6Can_GetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD *pValue);

//******************************************************************************
/// Set a DWORD configuration parameter.
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @param[in]    paramName        Parameter value to configure.\n
///                                Refer to the @ref scfg_can_imx6 for a list of supported parameters.
/// @param[in]    value            Data to be set
/// @param[in]    storageType      StoreToRegistry = parameter will also be stored in the registry\n
///                                StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_SetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD value, ParamStorageType storageType);

//******************************************************************************
/// Retrieves the current string configuration parameter.
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @param[in]    paramName        Pointer to a string containing the name of the string field to get.
///                                Refer to @ref scfg_can_imx6 for details.
/// @param[out]   pValue           Pointer to a buffer that receives the value's string data
/// @param[in]    maxBytes         Maximum size of string in bytes
/// @return                        Number of bytes read. 0 indicates an error.
DWORD Imx6Can_GetConfigString(HANDLE hPort, const TCHAR *paramName, TCHAR *pValue, DWORD maxBytes);

//******************************************************************************
/// Set string configuration parameters.\n
/// @param[in]    hPort            HANDLE received from Imx6Can_Init()
/// @param[in]    paramName        Pointer to a string containing the name of the string field to set.
///                                Refer to @ref scfg_can_imx6 for details.
/// @param[in]    pValue           String value to be set
/// @param[in]    storageType      StoreToRegistry = parameter will also be stored in the registry\n
///                                StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Imx6Can_SetConfigString(HANDLE hPort, const TCHAR *paramName, const TCHAR *pValue, ParamStorageType storageType);

//******************************************************************************
/// Get the revision of this CAN library
/// @param[out]    ver        library version, consisting of major, minor and build version
void Imx6Can_GetVersion(tVersion *ver);

#ifdef __cplusplus
}
#endif

#endif